/* $Id: separator.gen_h,v 1.30 2001/07/15 13:33:38 murrayc Exp $ */

/* separator.h
 *
 * Copyright (C) 1998-1999 The Gtk-- Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk--/widget.h>
#include <gtk/gtkseparator.h>
#include <gtk/gtkhseparator.h>
#include <gtk/gtkvseparator.h>


namespace Gtk {

//: A base class for {Gtk::HSeparator::} and {Gtk::Vseparator::}.
//- The {\class Gtk::Separator} widget is an abstract class, used only for deriving the
//- subclasses {Gtk::HSeparator::} and {Gtk::VSeparator::}.
class Separator : public Widget {


public:
  typedef Separator          CppObjectType;
  typedef GtkSeparator            BaseObjectType;

public:
  //: Returns the underlaying gtk+ object.
  GtkSeparator* gtkobj();
  const GtkSeparator* gtkobj() const;

  //: Returns true if object is this type.
  static bool isA(Gtk::Object *object);

  virtual ~Separator();

private:

public:



protected:
  Separator();



protected:
  // impl functions

};

//: Vertical line widget.
//- Filler widget used to separate widgets with a visible line.
//- This widget has no window and receives on configuration events.
//- It should be considered a leaf widget.
class VSeparator : public Separator {


public:
  typedef VSeparator          CppObjectType;
  typedef GtkVSeparator            BaseObjectType;

public:
  //: Returns the underlaying gtk+ object.
  GtkVSeparator* gtkobj();
  const GtkVSeparator* gtkobj() const;

  //: Returns true if object is this type.
  static bool isA(Gtk::Object *object);

  virtual ~VSeparator();

private:

public:
  VSeparator();





protected:
  // impl functions

};


//: Horizontal line widget.
//- Filler widget used to separate widgets with a visible line.
//- This widget has no window and receives no configuration events.
//- It should be considered a leaf widget.
class HSeparator : public Separator {


public:
  typedef HSeparator          CppObjectType;
  typedef GtkHSeparator            BaseObjectType;

public:
  //: Returns the underlaying gtk+ object.
  GtkHSeparator* gtkobj();
  const GtkHSeparator* gtkobj() const;

  //: Returns true if object is this type.
  static bool isA(Gtk::Object *object);

  virtual ~HSeparator();

private:

public:
  HSeparator();





protected:
  // impl functions

};


}
